""".. Ignore pydocstyle D400.

============
File finders
============

"""

import os

from django.apps import apps
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.module_loading import import_string


class BaseProcessesFinder:
    """Abstract processes loader for custom staticfiles finders."""

    def find_processes(self):
        """Abstract method."""
        raise NotImplementedError(
            "subclasses of BaseProcessesLoader must provide a find_processes() method"
        )

    def find_descriptors(self):
        """Abstract method."""
        raise NotImplementedError(
            "subclasses of BaseProcessesLoader must provide a find_descriptors() method"
        )


class FileSystemProcessesFinder(BaseProcessesFinder):
    """Find processes and descriptors in directory lists."""

    def find_processes(self):
        """Return a list of process directories."""
        return getattr(settings, "FLOW_PROCESSES_DIRS", ())

    def find_descriptors(self):
        """Return a list of descriptor directories."""
        return getattr(settings, "FLOW_DESCRIPTORS_DIRS", ())


class AppDirectoriesFinder(BaseProcessesFinder):
    """Find processes and descriptors in Django apps."""

    def _find_folders(self, folder_name):
        """Return a list of sub-directories."""
        found_folders = []
        for app_config in apps.get_app_configs():
            folder_path = os.path.join(app_config.path, folder_name)
            if os.path.isdir(folder_path):
                found_folders.append(folder_path)
        return found_folders

    def find_processes(self):
        """Return a list of process directories in the app."""
        return self._find_folders("processes")

    def find_descriptors(self):
        """Return a list of descriptor directories in the app."""
        return self._find_folders("descriptors")


def get_finders():
    """Get process finders."""
    for finder_path in settings.FLOW_PROCESSES_FINDERS:
        yield get_finder(finder_path)


def get_finder(import_path):
    """Get a process finder."""
    finder_class = import_string(import_path)
    if not issubclass(finder_class, BaseProcessesFinder):
        raise ImproperlyConfigured(
            'Finder "{}" is not a subclass of "{}"'.format(
                finder_class, BaseProcessesFinder
            )
        )
    return finder_class()



# Generated by Django 4.2 on 2023-05-10 11:09

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ("permissions", "0004_create_anonymous_user"),
        ("flow", "0012_alter_relation_category"),
    ]

    operations = [
        migrations.AlterField(
            model_name="annotationpreset",
            name="permission_group",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(class)s",
                to="permissions.permissiongroup",
            ),
        ),
        migrations.AlterField(
            model_name="collection",
            name="permission_group",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(class)s",
                to="permissions.permissiongroup",
            ),
        ),
        migrations.AlterField(
            model_name="data",
            name="permission_group",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(class)s",
                to="permissions.permissiongroup",
            ),
        ),
        migrations.AlterField(
            model_name="descriptorschema",
            name="permission_group",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(class)s",
                to="permissions.permissiongroup",
            ),
        ),
        migrations.AlterField(
            model_name="entity",
            name="permission_group",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(class)s",
                to="permissions.permissiongroup",
            ),
        ),
        migrations.AlterField(
            model_name="process",
            name="permission_group",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(class)s",
                to="permissions.permissiongroup",
            ),
        ),
        migrations.AlterField(
            model_name="relation",
            name="permission_group",
            field=models.ForeignKey(
                null=True,
                on_delete=django.db.models.deletion.CASCADE,
                related_name="%(class)s",
                to="permissions.permissiongroup",
            ),
        ),
    ]



"""Resolwe model database functions."""

from django.db.models.aggregates import Func

from .utils import json_path_components


class JsonGetPath(Func):
    """PostgreSQL JSON path (#>) operator."""

    function = "#>"
    template = "%(expressions)s%(function)s%%s"
    arity = 1

    def __init__(self, expression, path):
        """Initialize function.

        :param expression: Expression that returns a JSON field
        :param path: Path to get inside the JSON object, which can be
            either a list of path components or a dot-separated
            string
        """
        self.path = json_path_components(path)

        super().__init__(expression)

    def as_sql(self, compiler, connection):
        """Compile SQL for this function."""
        sql, params = super().as_sql(compiler, connection)
        params.append(self.path)
        return sql, params


class JsonbArrayElements(Func):
    """PostgreSQL jsonb_array_elements function."""

    function = "jsonb_array_elements"
    template = "%(function)s(%(expressions)s)"
    arity = 1
